@@ -66,6 +66,8 @@ namespace Imendio.Blam {
 
 	private uint              mAutoRefreshId;
 
+	private MessageConnection.MessageReceivedHandler mIpcHandler;
+	
 	enum TargetType {
 	    String,
 	    UriList
@@ -105,7 +107,8 @@ namespace Imendio.Blam {
 		Environment.Exit (0);
 	    }
 
-	    messageConn.SetCallback (new MessageConnection.MessageReceivedHandler (MessageReceivedCb));
+	    mIpcHandler = new MessageConnection.MessageReceivedHandler (MessageReceivedCb) ;
+	    messageConn.SetCallback (mIpcHandler);
 
 	    Proxy.InitProxy ();
             Utils.GeckoInit ();
@@ -130,7 +133,7 @@ namespace Imendio.Blam {
             PrepareGUI();
 
 	    Conf.AddNotify (new NotifyEventHandler (ConfNotifyHandler));
-            GLib.Idle.Add (new GLib.IdleHandler (IdleRefreshAll));
+	    //GLib.Idle.Add (new GLib.IdleHandler (IdleRefreshAll));
 	}
 	    
 	private void PrepareGUI()
@@ -288,6 +291,9 @@ namespace Imendio.Blam {
 		// Couldn't figure out how to get this to the menu item itself.
 		nextUnreadMenuItem.Activate ();
 		break;
+            case (Gdk.Key.Escape):
+                mainWindow.Iconify ();
+                break;
 	    }
 	}
 
@@ -597,15 +603,17 @@ namespace Imendio.Blam {
 	{
 	    int width, height;
 	    int position_x, position_y;
+	    string state;
 
+	    state = Conf.Get ("ui/main_window_state", "normal");
+        
 	    width = Conf.Get ("ui/main_window_width", 600);
-	    height = Conf.Get("ui/main_window_height", 400);
-
-	    mainWindow.Resize(width, height);
+	    height = Conf.Get ("ui/main_window_height", 400);
+	    
+	    mainWindow.Resize(width, height);	
 
 	    position_x = Conf.Get("ui/main_window_position_x", -1);
-	    position_y = Conf.Get("ui/main_window_position_y", -1);
-
+	    position_y = Conf.Get("ui/main_window_position_y", -1);                        
 	    if (position_x >= 0 && position_y >= 0) {
 		mainWindow.Move(position_x, position_y);
 	    }
@@ -614,26 +622,36 @@ namespace Imendio.Blam {
 	    channelPaned.Position = width;
 
 	    height = Conf.Get("ui/item_list_height", 100);
-	    itemPaned.Position = height;
+	    itemPaned.Position = height;	    
+
+	    if (state == "maximized") {
+	    	mainWindow.Maximize ();	    
+            }
 	}
 
 	private void SaveWindowState()
 	{
 	    int height, width;
 	    int position_x, position_y;
-
-	    mainWindow.GetSize(out width, out height);
 	    
-	    Conf.Set("ui/main_window_width", width);
-	    Conf.Set("ui/main_window_height", height);
+	    if (mainWindow.GdkWindow.State != Gdk.WindowState.Maximized) {
+	        Conf.Set ("ui/main_window_state", "normal");
+	    
+	        mainWindow.GetSize (out width, out height);
+	    
+	        Conf.Set ("ui/main_window_width", width);
+	        Conf.Set ("ui/main_window_height", height);
+	
+	        Conf.Set ("ui/channel_list_width", channelPaned.Position);
+	        Conf.Set ("ui/item_list_height", itemPaned.Position);
+	
+	        mainWindow.GetPosition (out position_x, out position_y);	    
 
-	    Conf.Set("ui/channel_list_width", channelPaned.Position);
-	    Conf.Set("ui/item_list_height", itemPaned.Position);
-
-	    mainWindow.GetPosition(out position_x, out position_y);
-
-	    Conf.Set("ui/main_window_position_x", position_x);
-	    Conf.Set("ui/main_window_position_y", position_y);
+	        Conf.Set ("ui/main_window_position_x", position_x);
+	        Conf.Set ("ui/main_window_position_y", position_y);	    
+	    } else {
+	    	Conf.Set ("ui/main_window_state", "maximized");	    
+	    }
 
 	    Conf.Sync ();
 	}
@@ -664,7 +682,7 @@ namespace Imendio.Blam {
 	    }
 
 	    mainWindow.Present ();
-	    mCollection.RefreshAll ();
+	    //mCollection.RefreshAll ();
 	}
 
         public static void Main(string[] args) 
