@@ -5,6 +5,8 @@
 // 
 
 using Rss;
+using Atom.Core;
+using Atom.Core.Collections;
 using System.Collections;
 using System;
 using System.Net;
@@ -19,6 +21,7 @@ namespace Imendio.Blam {
 	// Used when updating the feed
 	[XmlAttribute] public string LastModified = "";
 	[XmlAttribute] public string ETag = "";
+	[XmlAttribute] public string Type = "";
 
 	public int NrOfItems {
 	    get {
@@ -128,6 +131,23 @@ namespace Imendio.Blam {
                 return updated;
 	    }
 	}
+
+	public bool UpdateItem (string id, AtomEntry entry)
+	{
+	    Item item = GetItem (id);
+
+	    if (item == null) {
+	        item = new Item (id, entry);
+	        item.Channel = this;
+
+	        mItems.Add (item);
+	        return true;
+	    } else {
+	        bool updated = item.Update (entry);
+	        mUnupdatedItems.Remove (item);
+	        return updated;
+	    }
+	}
 	
 	/* Used to cross-mark as read */
 	public void MarkItemIdAsRead (string id)
@@ -175,6 +195,52 @@ namespace Imendio.Blam {
 	    this.PubDate = rssItem.PubDate;
 	}
 
+	public Item (string id, AtomEntry entry)
+	{
+	    this.Id = id;
+	    this.Title = entry.Title.Content.Trim();
+
+	    if (entry.Author != null && entry.Author.Name != "") {
+	        this.Author = entry.Author.Name.Trim();
+	    }
+
+	    /* Content is optional in Atom feeds, or there may be multiple 
+	     * contents of different media types.  Use the summary if there 
+	     * is no content, otherwise use HTML if it's available.
+	     */
+	    if (entry.Contents == null || entry.Contents.Count == 0) {
+	        this.Text = entry.Summary.Content.Trim();
+	    } else {
+	        foreach (AtomContent content in entry.Contents) {
+		    if (content.Type == MediaType.TextHtml || 
+			content.Type == MediaType.ApplicationXhtmlXml) {
+		          this.Text = content.Content.Trim();
+		          break;
+		    }
+	        }
+	    }
+
+	    /* Atom entries must have at least one link with a relationship 
+	     * type of "alternate," but may have more.  Again, prefer HTML if 
+	     * more than one is available.
+	     */
+	    if (entry.Links.Count == 1) {
+	      this.Link = entry.Links[0].HRef.ToString().Trim();
+	    } else {
+	        foreach (AtomLink link in entry.Links) {
+	            if (link.Rel == Relationship.Alternate) {
+		        if (link.Type == MediaType.TextHtml || 
+			    link.Type == MediaType.ApplicationXhtmlXml) {
+		            this.Link = link.HRef.ToString().Trim();
+		            break;
+		        }
+	            }
+	        }
+	    }
+
+	    this.PubDate = entry.Modified.DateTime;
+	}
+
 	/* This is called in the middle of an refresh so the channel will be
 	 * updated when refresh is done
 	 */
@@ -193,6 +259,34 @@ namespace Imendio.Blam {
 	    return false;
 	}
 
+	public bool Update (AtomEntry entry)
+	{
+	    if (this.Title.Equals(entry.Title.Content.Trim()) == false) {
+	        this.Title = entry.Title.Content.Trim();
+	        return true;
+	    }
+
+	    string entryText = "";
+	    if (entry.Contents == null || entry.Contents.Count == 0) {
+	        entryText = entry.Summary.Content.Trim();
+	    } else {
+	        foreach (AtomContent content in entry.Contents) {
+	            if (content.Type == MediaType.TextHtml ||
+		        content.Type == MediaType.ApplicationXhtmlXml) {
+		        entryText = content.Content.Trim();
+		        break;
+	            }
+	        }
+	    }
+
+	    if (this.Text.Equals(entryText) == false) {
+	        this.Text = entryText;
+	        return true;
+	    }
+
+	    return false;
+	}
+
 	public void SetUnread (bool unread, bool inAllChannels) 
 	{
 	    if (Unread != unread) {
